/* -*- coding: utf-8 -*- */
#include "helper_fns.h"
#include "joy_data.h"

extern int verbosity;

void init_values(int n_joys, int n_axes, int n_hats, int n_btns) {
  int i , j = 0, id, btn_id, axis_id = 0;

  if (verbosity > 1) {
    printf("NUM_JOYS: %d, MAX_AXIS: %d, MAX_HAT: %d, MAX_BTN: %d\n",
           n_joys, n_axes, n_hats, n_btns);
  }

  for (i = 0; i < n_joys; i++) {
    id = i;

    /* init buttons */
    for (j = 0; j < n_btns; j++) {
      btn_id = j;
      btns[id][btn_id].joy_id = id;
      btns[id][btn_id].button = btn_id;
      btns[id][btn_id].keysym = 0;
      btns[id][btn_id].type = SDL_JOYBUTTONDOWN;
    }

    /* init axes */
    for (j = 0; j < n_axes; j++) {
      axis_id = j;
      axes[id][axis_id].joy_id = id;
      axes[id][axis_id].axis_id = axis_id;
      if (axis_id % 2 == 0) { axes[id][axis_id].axis = 'x'; }
      else { axes[id][axis_id].axis = 'y'; }
      axes[id][axis_id].min_range = -1;
      axes[id][axis_id].max_range = 1;
      axes[id][axis_id].threshold = 10000;
      if (axis_id % 2 == 0) {
        axes[id][axis_id].keylow = 100;
        axes[id][axis_id].keyhigh = 102;
      }
      else {
        axes[id][axis_id].keylow = 98;
        axes[id][axis_id].keyhigh = 104;
      }
      axes[id][axis_id].type = SDL_JOYAXISMOTION;
      axes[id][axis_id].flushCounter = 0;
    }

    /* init hats */
    for (j = 0; j < n_hats; j++) {
      axis_id = j;
      hats[id][axis_id].joy_id = id;
      hats[id][axis_id].axis_id = axis_id;
      if (axis_id % 2 == 0) { axes[id][axis_id].axis = 'x'; }
      else { axes[id][axis_id].axis = 'y'; }
      hats[id][axis_id].min_range = -1;
      hats[id][axis_id].max_range = 1;
      hats[id][axis_id].threshold = 10000;
      if (axis_id % 2 == 0) {
        hats[id][axis_id].keylow = 100;
        hats[id][axis_id].keyhigh = 102;
      }
      else {
        hats[id][axis_id].keylow = 98;
        hats[id][axis_id].keyhigh = 104;
      }
      hats[id][axis_id].type = SDL_JOYHATMOTION;
      hats[id][axis_id].flushCounter = 0;
    }
  }
}

void alloc_values() {
  int i = 0, MAX_AXIS = 0, MAX_HAT = 0, MAX_BTN = 0;
  smooth_refresh = 3;
  smooth_algorithm = 0;

  NUM_JOYS = SDL_NumJoysticks();

  joys = (SDL_Joystick**)malloc(NUM_JOYS * sizeof(SDL_Joystick*));
  dev_map = (int*)malloc(NUM_JOYS * sizeof(int));
  cfg_map = (int*)malloc(NUM_JOYS * sizeof(int));

  /* this should be safe, because non joysticks contain 0 axes/hats/buttons */
  for (i = 0; i < NUM_JOYS; i++) {
    SDL_Joystick *joy = SDL_JoystickOpen(i);
    MAX_AXIS = max(SDL_JoystickNumAxes(joy), MAX_AXIS);
    /* we multiply the number of hats by two, since we treat hats like they are
       two separate axes, even though in SDL they are not; it is merely an
       abstraction */
    MAX_HAT =  max(SDL_JoystickNumHats(joy), MAX_HAT) * 2;
    MAX_BTN =  max(SDL_JoystickNumButtons(joy), MAX_BTN);
    SDL_JoystickClose(joy);
  }
  axes = (struct joy_axis **)malloc(NUM_JOYS * sizeof(struct joy_axis *));
  *axes = (struct joy_axis *)
    malloc(MAX_AXIS * NUM_JOYS * sizeof(struct joy_axis));
  hats = (struct joy_hat **)malloc(NUM_JOYS * sizeof(struct joy_hat *));
  *hats = (struct joy_hat *)
    malloc(MAX_HAT * NUM_JOYS * sizeof(struct joy_hat));
  btns = (struct joy_button **)malloc(NUM_JOYS * sizeof(struct joy_button *));
  *btns = (struct joy_button *)
    malloc(MAX_BTN * NUM_JOYS * sizeof(struct joy_button));

  for (i = 0; i < NUM_JOYS; i++) { axes[i] = *axes + (i * MAX_AXIS); }
  for (i = 0; i < NUM_JOYS; i++) { hats[i] = *hats + (i * MAX_HAT); }
  for (i = 0; i < NUM_JOYS; i++) { btns[i] = *btns + (i * MAX_BTN); }

  init_values(NUM_JOYS, MAX_AXIS, MAX_HAT, MAX_BTN);
}

void dealloc_values() {
  free(*axes); free(axes);
  free(*hats); free(hats);
  free(*btns); free(btns);
  free(joys);
  free(dev_map);
  free(cfg_map);
}
