/* -*- coding: utf-8 -*- */
#include <stdio.h>
#include <unistd.h>
#include <X11/Xlib.h>
#include "helper_fns.h"
#include "handle_args.h"
#include "joy_devices.h"
#include "joy_data.h"
#include "alloc_values.h"
#include "read_values.h"
#include "detect_joysticks.h"

#include "event_loop.h"

Display* d;

int verbosity = 0;

char filename[255];
char local_filename[255];
char *profile;
int want_daemon = -1;
int list_devices = 0;
int list_sdl_devices = 0;
SDL_Event event;

int main(int argc, char **argv) {
  int config_file_type;
  d = XOpenDisplay(NULL);

  handle_args(argc, argv);
  config_file_type = handle_config(filename, local_filename);

  if (verbosity > 0) {
    if (config_file_type == Local)
      printf("using profile in file: %s\n", local_filename);
    else if (config_file_type == System_Wide)
      printf("using profile in file: %s\n", filename);
  }

  if (want_daemon == 1) {
    if (fork() == 0) { if (verbosity > 0) { puts("running in daemon"); } }
    else { puts("failed to fork process"); exit(Fail); }
  }
  else { if (verbosity >= 0) { puts("running in background"); } }

  if (d == NULL) { return 1; }
  else {
    if (verbosity >= 0) { printf("Initializing SDL.\n"); }

    /* SDL still needs video to be initialized to detect joystick events */
    if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_JOYSTICK) == Fail) {
      printf("Could not initialize SDL: %s.\n", SDL_GetError());
      exit(Fail);
    }

    if (verbosity >= 0) { puts("SDL initialized."); }

    alloc_values();
    if (config_file_type == Local) { read_values(local_filename); }
    else if (config_file_type == System_Wide) { read_values(filename); }
    else { puts("Can't find a suiteable configuration file."); exit(Fail); }
    detect_joysticks();

    SDL_JoystickEventState(SDL_ENABLE);

    /* main event loop, process joystick requests as they arrive */
    event_loop(event);

    if (verbosity >= 0) { puts("Deallocating Structures"); }
    dealloc_values();

    if (verbosity >= 0) { puts("Quiting SDL"); }
    SDL_Quit();
    if (verbosity >= 0) { puts("Quiting...."); }
    exit(0);
  }
}
