/* -*- coding: utf-8 -*- */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "joy_data.h"

extern struct joy_axis** axes;
extern struct joy_hat** hats;
extern struct joy_button** btns;

extern int verbosity;

void test_configuration() {
  int i;
  for (i = 0; i < NUM_JOYS; i++) {
    if (axes[i][1].axis_id != 0) { cfg_map[i] = 1;}
    else if (axes[i][1].axis_id == 0 && hats[i][1].axis_id == 0) {
      cfg_map[i] = -1;
    }
    else {
      puts("error, config file invalid");
      exit(-1);
    }
  }
}

int read_values(char* filename) {
  int line_num = 0, id, axis_id, btn_id, i = 0, tokenize = 0;

  /* open file for reading */
  fn = fopen (filename, "r");

  while (!feof(fn)) {
    char* stype = NULL;
    char line[150];
    char* vals[20];

    i = 0;

    fgets(line, sizeof line, fn);

    while (strcmp(line, "\n") == 0) {
      puts("warning: empty line found in profile");
      fgets(line, sizeof line, fn);
      if (feof(fn)) { break; }
    }

    if (feof(fn)) { break; }

    line_num++;

    tokenize = 1;

    if (strcmp(line, "\n") != 0 && line[0] == '#') {
      if (verbosity > 1)
        puts("skip commented line");
      tokenize = 0;
    }
    else if (strcmp(line, "\n") != 0 && line[0] == ' ') {
      puts("warning: line with heading whitespace found in profile");
    }
    if (tokenize == 1) {
      char *tokenPtr = NULL;

      tokenPtr = strtok(line, " ");
      vals[i] = tokenPtr;

      while (tokenPtr != NULL) {
        i++;
        tokenPtr = strtok(NULL, ", ");
        vals[i] = tokenPtr;
      }

      stype = vals[0];
      id = atoi(vals[2]);

      if (id < NUM_JOYS) {
        if (strcmp(stype, "setting") == 0) {
          if (strcmp(vals[1], "smooth_refresh:") == 0) {
            smooth_refresh = atoi(vals[2]);
            if (verbosity > 0)
              printf("setting smooth_refresh: %d\n", smooth_refresh);
          }
          else if (strcmp(vals[1], "smooth_algorithm:") == 0) {
            smooth_algorithm = atoi(vals[2]);
            if (verbosity > 0)
              printf("setting smooth_algorithm: %d\n", smooth_algorithm);
          }
        }
        else if (strcmp(stype, "axis") == 0) {
          axis_id = atoi(vals[4]);
          axes[id][axis_id].joy_id = id;
          axes[id][axis_id].axis_id = axis_id;
          axes[id][axis_id].axis = vals[6][0];
          axes[id][axis_id].min_range = atoi(vals[8]);
          axes[id][axis_id].max_range = atoi(vals[10]);
          axes[id][axis_id].threshold = atoi(vals[12]);
          axes[id][axis_id].keylow = atoi(vals[14]);
          axes[id][axis_id].keyhigh = atoi(vals[16]);
          axes[id][axis_id].type = SDL_JOYAXISMOTION;
        }
        else if (strcmp(stype, "hat") == 0) {
          axis_id = atoi(vals[4]);
          hats[id][axis_id].joy_id = id;
          hats[id][axis_id].axis_id = axis_id;
          hats[id][axis_id].axis = vals[6][0];
          hats[id][axis_id].min_range = atoi(vals[8]);
          hats[id][axis_id].max_range = atoi(vals[10]);
          hats[id][axis_id].threshold = atoi(vals[12]);
          hats[id][axis_id].keylow = atoi(vals[14]);
          hats[id][axis_id].keyhigh = atoi(vals[16]);
          hats[id][axis_id].type = SDL_JOYHATMOTION;
        }
        else if (strcmp(stype, "button") == 0) {
          btn_id = atoi(vals[4]);
          btns[id][btn_id].joy_id = id;
          btns[id][btn_id].button = btn_id;
          btns[id][btn_id].keysym = atoi(vals[6]);
          btns[id][btn_id].type = SDL_JOYBUTTONDOWN;
        }
        else if (stype == NULL) { puts("invalid configuration file\n"); }
        else {
          printf("invalid event type %s in configuration file", stype);
          return -1;
        }
      }
      else {
        if (verbosity > 1)
          printf("ignoring line related to joystick %d in profile\n", id);
      }
    }
  }
  test_configuration();
  return 0;
}
